'
'  svnpropertylog - v1.1 - 05/02/2016
'  Copyright (C) 2016 Antoine Beauchamp
'  The code & updates for the library can be found on http://end2endzone.com
'
' AUTHOR/LICENSE:
'  This library is free software; you can redistribute it and/or
'  modify it under the terms of the GNU Lesser General Public
'  License as published by the Free Software Foundation; either
'  version 3.0 of the License, or (at your option) any later version.
'
'  This library is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY; without even the implied warranty of
'  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'  Lesser General Public License (LGPL-3.0) for more details.
'
'  You should have received a copy of the GNU Lesser General Public
'  License along with this library; if not, write to the Free Software
'  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
'
' DISCLAIMER:
'  This software is furnished "as is", without technical support, and with no 
'  warranty, express or implied, as to its usefulness for any purpose.
'
' PURPOSE:
'  The subversion application does not offer native support for identifying
'  changes to properties as easily as changes to files. The purpose of
'  this script is to allows a subversion user to find all revisions where
'  a change to a specific property occured.
'
'  Calling the script finds all revisions where a given property has been changed.
'  Usage:
'   Argument 0: Oldest revision number
'   Argument 1: Newest revision number
'   Argument 2: File or folder URL
'   Argument 3: Property name
'   Argument 4: Show property values? ( Expecting 0 or 1 )
'
' HISTORY:
' 04/30/2016 v1.0 - Initial release.
' 05/02/2016 v1.1 - Small correction to license file & header.
'

Option Explicit

'DebugArguments()

'Validate arguments
if WScript.Arguments.Count < 4 then
  WScript.Echo "Error missing argument. See Usage for details."
  DebugArguments()
  WScript.Quit
end if

'Extract command line arguments
dim svnNewestRevision
dim svnOldestRevision
dim svnUrl
dim svnPropertyName
svnNewestRevision = cint(WScript.Arguments.Item(0))
svnOldestRevision  = cint(WScript.Arguments.Item(1))
svnUrl             =      WScript.Arguments.Item(2)
svnPropertyName    =      WScript.Arguments.Item(3)

'Detect showing or not property values
dim showPropertyValues
showPropertyValues = False
if WScript.Arguments.Count = 5 then
  if WScript.Arguments.Item(4) = "1" Then showPropertyValues = True
end if

'Automatically detect incorrect revision order
if svnNewestRevision < svnOldestRevision then
  'swap
  dim tmp
  tmp = svnNewestRevision
  svnNewestRevision = svnOldestRevision
  svnOldestRevision = tmp
end if

'Find a temp filename
dim tempFilename
tempFilename = GetApplicationTempFile()

'Define a list for storing each revision's 
dim propertyValues
Set propertyValues = CreateObject("System.Collections.ArrayList")

'Extract property to values for each revision
dim progress
progress = 0
WScript.Echo "svnpropertylog v1.1"
WScript.Echo "Extracting property " & svnPropertyName & " within revision " & svnOldestRevision & " to " & svnNewestRevision & " from " & svnUrl & "..."

dim loopCount
loopCount = 0

dim i
for i = svnNewestRevision to svnOldestRevision step -1
  dim commandline
  commandLine = "cmd /c svn -r" & i & " propget """ & svnPropertyName & """ """ & svnUrl & """"
  'msgbox commandLine

  'Launch the command line program and capture the output to propertyValue
  dim propertyValue
  propertyValue = vFn_Sys_Run_CommandOutput(commandLine, 1, 0, tempFilename, 1, 1)
  'msgbox propertyValue

  'Add the property value of current revision the propertyValues list
  propertyValues.Add propertyValue

  'Show progress ?
  loopCount = loopCount + 1
  if loopCount > (svnNewestRevision-svnOldestRevision)/20 then 'every 5%
    loopCount = 0

    'Update progress   
    progress = toProgress(i, svnNewestRevision, svnOldestRevision)
    WScript.Echo "Extracting... " & progress & "%"
  end if

next

'Compare property values
WScript.Echo "Comparing properties..."
dim nextRevisionPropertyValue
nextRevisionPropertyValue = propertyValues.Item(0)
for i = svnNewestRevision to svnOldestRevision step -1
  dim listIdx
  listIdx = toListIndex(i, svnNewestRevision, svnOldestRevision)

  dim currentRevisionPropertyValue
  currentRevisionPropertyValue = propertyValues.Item(listIdx)

  if currentRevisionPropertyValue <> nextRevisionPropertyValue then
    WScript.Echo "Property " & svnPropertyName & " has been changed in revision " & i+1
    if showPropertyValues then
      WScript.Echo "==============================================================================="
      WScript.Echo "revision: " & i+1
      WScript.Echo propertyValues.Item(toListIndex(i+1, svnNewestRevision, svnOldestRevision))
      WScript.Echo "==============================================================================="
    end if
    nextRevisionPropertyValue = currentRevisionPropertyValue
  end if
next

Set propertyValues = Nothing







'Convert loop i to list index
Function toListIndex(i, maxValue, minValue)
  dim totalsteps
  totalsteps = (maxValue-minValue)
  dim position
  position = i-minValue 'from totalsteps to 0
  position = (-1*position) + totalsteps
  toListIndex = position
End Function

'Convert loop i to a progress % from 0 to 100
Function toProgress(i, maxValue, minValue)
  dim totalsteps
  totalsteps = (maxValue-minValue)
  dim position
  position = i-minValue 'from totalsteps to 0
  position = (-1*position) + totalsteps
  dim progress
  progress = (position * 100) / totalsteps
  progress = cint(progress)
  toProgress = progress
End Function

Function GetApplicationTempFile()
  dim objFSO
  Set objFSO = CreateObject("Scripting.FileSystemObject")

  dim tempDir
  tempDir = objFSO.GetSpecialFolder(2)

  dim filename
  filename = objFSO.GetFileName(WScript.ScriptFullName)
  filename = filename & ".tmp"

  dim tempFilenamePath
  tempFilenamePath = tempDir & "\" & filename

  GetApplicationTempFile = tempFilenamePath

  Set objFSO = Nothing
End Function

Sub DebugArguments()
  Dim i
  WScript.Echo "Found " & WScript.Arguments.Count & " arguments."
  For i = 0 To WScript.Arguments.Count - 1
    WScript.Echo i & "=" & WScript.Arguments.Item(i)
  Next
End Sub

Function vFn_Sys_Run_CommandOutput (Command, Wait, Show, OutToFile, DeleteOutput, NoQuotes)
  'http://stackoverflow.com/questions/5690134/running-command-line-silently-with-vbscript-and-getting-output
  'Run Command similar to the command prompt, for Wait use 1 or 0. Output returned and
  'stored in a file.
  'Command = The command line instruction you wish to run.
  'Wait = 1/0; 1 will wait for the command to finish before continuing.
  'Show = 1/0; 1 will show for the command window.
  'OutToFile = The file you wish to have the output recorded to.
  'DeleteOutput = 1/0; 1 deletes the output file. Output is still returned to variable.
  'NoQuotes = 1/0; 1 will skip wrapping the command with quotes, some commands wont work
  '                if you wrap them in quotes.
  '----------------------------------------------------------------------------------------
  dim f_objShell
  dim f_objFso
  dim tCommand
  dim tOutToFile
  dim f_objFile
  dim tMyOutput
  dim tWait
  dim tShow
  'On Error Resume Next
  On Error Goto 0
  Set f_objShell = CreateObject("Wscript.Shell")
  Set f_objFso = CreateObject("Scripting.FileSystemObject")
  Const ForReading = 1, ForWriting = 2, ForAppending = 8
  'VARIABLES
  If OutToFile = "" Then OutToFile = "TEMP.TXT"
  tCommand = Command
  If Left(Command,1)<>"""" And NoQuotes <> 1 Then tCommand = """" & Command & """"
  tOutToFile = OutToFile
  If Left(OutToFile,1)<>"""" Then tOutToFile = """" & OutToFile & """"
  If Wait = 1 Then tWait = True
  If Wait <> 1 Then tWait = False
  If Show = 1 Then tShow = 1
  If Show <> 1 Then tShow = 0
  'RUN PROGRAM
  f_objShell.Run tCommand & ">" & tOutToFile, tShow, tWait
  'READ OUTPUT FOR RETURN
  Set f_objFile = f_objFso.OpenTextFile(OutToFile, 1)
  if (f_objFile.AtEndOfStream) then
    tMyOutput = ""
  else
    tMyOutput = f_objFile.ReadAll
  end if
  f_objFile.Close
  Set f_objFile = Nothing
  'DELETE FILE AND FINISH FUNCTION
  If DeleteOutput = 1 Then
    Set f_objFile = f_objFso.GetFile(OutToFile)
    f_objFile.Delete
    Set f_objFile = Nothing
  End If
  vFn_Sys_Run_CommandOutput = tMyOutput
  If Err.Number <> 0 Then vFn_Sys_Run_CommandOutput = "<0>"
  Err.Clear
  On Error Goto 0
  'Set f_objShell = Nothing
End Function
